#include "bookshelf.h"
#include <cstdint>
#include <vector>
#include <iostream>
#include <random>
#include <utility>


class Node {
	static std::mt19937 rand;
	Node *l, *r;
	const uint32_t p;

	uint32_t subTree;

	void reeval() {
		subTree = 1;
		subTree += l ? l->subTree : 0;
		subTree += r ? r->subTree : 0;
	}

public:
	uint32_t val;

	Node(uint32_t val = 0): val(val), l(nullptr), r(nullptr), p(rand()),
		subTree(1) {}

	static Node* merge(Node *l, Node *r);
	static std::pair<Node*, Node*> split(Node *n, uint32_t lCnt);
	static Node* insert(Node *root, uint32_t val, uint32_t pos);

	void get_vals(std::vector<uint32_t> &res);
};

std::mt19937 Node::rand(0xCAFEBABE);
Node* Node::merge(Node *l, Node *r) {
	if ((l == nullptr) || (r == nullptr)) {
		return l ? l : r;
	}
	if (l->p > r->p) {
		l->r = merge(l->r, r);
		l->reeval();
		return l;
	} else {
		r->l = merge(l, r->l);
		r->reeval();
		return r;
	}
}

std::pair<Node*, Node*> Node::split(Node *n, uint32_t lCnt) {
	if (n == nullptr) {
		return {nullptr, nullptr};
	}
	uint32_t lSubtree = n->l ? n->l->subTree : 0;
	if (lCnt <= lSubtree) {
		auto t = split(n->l, lCnt);
		n->l = t.second;
		n->reeval();
		return {t.first, n};
	} else {
		auto t = split(n->r, lCnt - lSubtree - 1);
		n->r = t.first;
		n->reeval();
		return {n, t.second};
	}
}

Node* Node::insert(Node *root, uint32_t val, uint32_t i) {
	Node *n = new Node(val);
	auto s = Node::split(root, i);
	return merge(s.first, merge(n, s.second));
}


void Node::get_vals(std::vector<uint32_t> &res) {
	if (l) {
		l->get_vals(res);
	}
	res.push_back(val);
	if (r) {
		r->get_vals(res);
	}
}


int64_t curInv;
uint32_t n;

Node *treap = nullptr;

void build_tree() {
	treap = Node::insert(treap, 1, 0);
	int64_t curInd = 0;

	for (uint32_t i = 1;i < n;i++) {
		int64_t nInv = bookswap(1, i + 1);
		curInv--; //???
//		if (nInv < curInv) {
//			curInv--;
//		}
		int64_t nInd = nInv - curInv + 2 * curInd;
		//std::cerr << curInv << " " << nInv << " "<< nInd << std::endl;
		nInd /= 2;
		//std::cerr << nInd << "\n";
		treap = Node::insert(treap, i + 1, nInd);
		curInd = nInd;
		curInv = nInv;

//		std::vector<uint32_t> v;
//		treap->get_vals(v);
//		for (uint32_t i = 0;i < v.size();i++) {
//			std::cerr << v[i] << " ";
//		}
//		std::cerr << std::endl;

	}

}

const uint32_t MAX_N = 2e5;

int32_t ans[MAX_N];

void recover_ans() {
	std::vector<uint32_t> v;
	treap->get_vals(v);

	for (uint32_t i = 0;i < v.size();i++) {
		//std::cerr << v[i] << " ";
		ans[v[i]] = i + 1;
	}
	std::cerr << "\n";

}

void bookshelf(int N, long long Inv) {
	n = N;
	curInv = Inv;
	build_tree();
	recover_ans();
	answer(ans);
}

