#include <bits/stdc++.h>
#define endl '\n'
using namespace std;

const int32_t MAX_ROWS = 1e5 + 5;

class SegmentTree {
private:
	class Node {
	public:	
		int32_t low, high;
		int64_t val, lazy1;
		Node *left, *right;
		queue< pair< int32_t, int32_t > > lazy2;
	
		Node(int32_t low, int32_t high) : low(low), high(high) {
			val = 0;
			lazy1 = 0;
			left = nullptr;
			right = nullptr;
		}
	};

	Node *root;
	
	void PushLazy(Node *nd) {
		if(nd->lazy1 == 0 && nd->lazy2.empty()) {
			return;
		}

		int32_t low = nd->low, high = nd->high;
		int32_t mid = (low + high) / 2;
		
		nd->val += 1ll * (high - low + 1) * nd->lazy1;
		nd->left->lazy1 += nd->lazy1;
		nd->right->lazy1 += nd->lazy1;
		nd->lazy1 = 0;

		while(!nd->lazy2.empty()) {
			int32_t st = nd->lazy2.front().first;
			int32_t d = nd->lazy2.front().second;
			nd->lazy2.pop();

			int32_t en = st + high - low;

			int64_t sum = 1ll * (en + 1) * en / 2 - 1ll * st * (st - 1) / 2;
			nd->val += sum * d;

			nd->left->lazy2.push({ st, d });
			nd->right->lazy2.push({ st + (mid - low) + 1, d});
		}
	}

	void Update(Node *nd, int32_t qLow, int32_t qHigh, int32_t st, int32_t d) {
		int32_t low = nd->low, high = nd->high;
		if(low > qHigh || high < qLow) {
			return;
		}
		else if(low >= qLow && high <= qHigh) {
			int32_t aux = low - qLow;
			nd->lazy1 += 1ll * st;
			nd->lazy2.push({ aux, d });
			return;
		}

		int32_t mid = (low + high) / 2;
		if(nd->left == nullptr) {
			nd->left = new Node(low, mid);
		}
		if(nd->right == nullptr) {
			nd->right = new Node(mid + 1, high);
		}

		PushLazy(nd);
		Update(nd->left, qLow, qHigh, st, d);
		Update(nd->right, qLow, qHigh, st, d);

		nd->val = nd->left->val + nd->right->val;
	}

	int64_t Query(Node *nd, int32_t qLow, int32_t qHigh) {
		int32_t low = nd->low, high = nd->high;

		if(low > qHigh || high < qLow) {
			return 0;
		}

		int32_t mid = (low + high) / 2;
		if(nd->left == nullptr) {
			nd->left = new Node(low, mid);
		}
		if(nd->right == nullptr) {
			nd->right = new Node(mid + 1, high);
		}
		
		PushLazy(nd);
		
		if(low >= qLow && high <= qHigh) {
			return nd->val;
		}

		return Query(nd->left, qLow, qHigh) + Query(nd->right, qLow, qHigh);
	}

public:
	void Init(int32_t rowSize) {
		root = new Node(1, rowSize);
	}

	void Update(int32_t low, int32_t high, int32_t st, int32_t d) {
		Update(root, low, high, st, d);
	}

	int64_t Query(int32_t low, int32_t high) {
		return Query(root, low, high);
	}
};

SegmentTree segTree/*s[MAX_ROWS]*/;
//unordered_map< int32_t, bool > initedRows;

int main() {
	ios_base::sync_with_stdio(false);
	cin.tie(nullptr);

	int32_t n, q;
	cin >> n >> q;

	segTree.Init(n);
	for(int32_t i = 1; i <= q; i++) {
		int32_t type;
		cin >> type;

		if(type == 1) {
			int32_t row, low, high, st, d;
			cin >> row >> low >> high >> st >> d;
			
			/*
			if(!initedRows[row]) {
				initedRows[row] = true;
				segTrees[row].Init(n);
			}
			*/

			segTree/*s[row]*/.Update(low, high, st, d);
		}
		else if(type == 2) {
			int32_t row, low, high;
			cin >> row >> low >> high;

			cout << segTree/*s[row]*/.Query(low, high) << endl;
		}
	}

	return 0;
}
